@@ -1,22 +1,20 @@
 import os
-import shutil
 import threading
 from tempfile import NamedTemporaryFile
-from typing import Generator, List, Union
+from typing import Generator, List, Optional, Union
 
-import deltachat as dc
+import deltachat as dc  # type: ignore
 import py
-from deltachat import Account, Chat, Contact, Message, account_hookimpl, const
-from deltachat.capi import ffi, lib
-from deltachat.cutil import as_dc_charpointer
-from deltachat.message import parse_system_add_remove
-from deltachat.tracker import ConfigureTracker
+from deltachat import Account, Chat, Contact, Message, account_hookimpl
+from deltachat.message import parse_system_add_remove  # type: ignore
+from deltachat.tracker import ConfigureTracker  # type: ignore
 
 from .builtin.admin import add_admin, del_admin, get_admins
 from .builtin.cmdline import PluginCmd
 from .commands import Commands, _cmds
 from .filters import Filters, _filters
 from .plugins import Plugins, get_global_plugin_manager
+from .templates import help_template
 from .utils import (
     parse_system_image_changed,
     parse_system_title_changed,
@@ -43,7 +41,7 @@ class Replies:
         bytefile=None,
         sender: str = None,
         quote: Message = None,
-        chat: Chat = None
+        chat: Chat = None,
     ) -> None:
         """Schedule a reply message.
 
@@ -75,7 +73,7 @@ class Replies:
     def send_reply_messages(self) -> list:
         l = []
         for msg in self._send_replies_to_core():
-            self.logger.info(
+            self.logger.debug(
                 "reply id={} chat={} sent with text: {!r}".format(
                     msg.id, msg.chat, msg.text[:50]
                 )
@@ -115,6 +113,7 @@ class Replies:
         quote: Message = None,
     ) -> Message:
         if bytefile:
+            assert filename is not None, "bytefile given but filename not provided"
             blobdir = self.incoming_message.account.get_blobdir()
             parts = filename.split(".", maxsplit=1)
             if len(parts) == 2:
@@ -131,41 +130,23 @@ class Replies:
             with open(filename, "wb") as f:
                 f.write(bytefile.read())
 
-        _view_type_mapping = {
-            "text": const.DC_MSG_TEXT,
-            "image": const.DC_MSG_IMAGE,
-            "gif": const.DC_MSG_GIF,
-            "audio": const.DC_MSG_AUDIO,
-            "video": const.DC_MSG_VIDEO,
-            "file": const.DC_MSG_FILE,
-            "sticker": const.DC_MSG_STICKER,
-        }
         if not viewtype:
             if filename:
                 viewtype = "file"
             else:
                 viewtype = "text"
-        view_type_code = _view_type_mapping.get(viewtype, viewtype)
-        msg = Message(
-            self.incoming_message.account,
-            ffi.gc(
-                lib.dc_msg_new(
-                    self.incoming_message.account._dc_context, view_type_code
-                ),
-                lib.dc_msg_unref,
-            ),
-        )
+        msg = Message.new_empty(self.incoming_message.account, viewtype)
 
         if quote is not None:
             msg.quote = quote
         if text:
             msg.set_text(text)
         if html:
-            lib.dc_msg_set_html(msg._dc_msg, as_dc_charpointer(html))
+            msg.set_html(html)
         if filename:
             msg.set_file(filename)
         if sender:
-            lib.dc_msg_set_override_sender_name(msg._dc_msg, as_dc_charpointer(sender))
+            msg.set_override_sender_name(sender)
 
         return msg
 
@@ -203,6 +184,10 @@ class DeltaBot:
         plugin_manager.hook.deltabot_init.call_historic(
             kwargs=dict(bot=self, args=args)
         )
+
+        # register /help command
+        self.commands.register(func=self._help, name="/help")
+
         # add manually added python modules as plugins
         mods = self.get(PluginCmd.db_key, "").split("\n")
         while mods:
@@ -221,11 +206,11 @@ class DeltaBot:
                 else:
                     self.logger.warning("Plugin not found: %s", pymodule)
 
-        for args in _cmds:
-            self.commands.register(*args)
+        for items in _cmds:
+            self.commands.register(**dict(items))
 
-        for args in _filters:
-            self.filters.register(*args)
+        for items in _filters:
+            self.filters.register(**dict(items))
 
     #
     # API for bot administration
@@ -264,7 +249,7 @@ class DeltaBot:
     # API for persistent scoped-key/value settings
     #
     def set(self, name: str, value: str, scope: str = "global") -> str:
-        """ Store a bot setting with the given scope. """
+        """Store a bot setting with the given scope."""
         assert "/" not in scope and "/" not in name
         old_val = self.get(name, scope=scope)
         key = scope + "/" + name
@@ -272,13 +257,15 @@ class DeltaBot:
         return old_val
 
     def delete(self, name: str, scope: str = "global") -> None:
-        """ Delete a bot setting with the given scope. """
+        """Delete a bot setting with the given scope."""
         assert "/" not in scope
         key = scope + "/" + name
         self.plugins._pm.hook.deltabot_store_setting(key=key, value=None)
 
-    def get(self, name: str, default: str = None, scope: str = "global") -> str:
-        """ Get a bot setting from the given scope. """
+    def get(
+        self, name: str, default: str = None, scope: str = "global"
+    ) -> Optional[str]:
+        """Get a bot setting from the given scope."""
         assert "/" not in scope
         key = scope + "/" + name
         res = self.plugins._pm.hook.deltabot_get_setting(key=key)
@@ -300,12 +287,32 @@ class DeltaBot:
             ]
         return l
 
+    def add_preference(self, name: str, description: str) -> None:
+        """add a preference users can set with /set command.
+
+        The given description will be shown to the users.
+        """
+        assert description is not None
+        self.set(name, description, scope="preferences")
+
+    def get_preference_description(self, name: str) -> Optional[str]:
+        """Return preference description or None if the preference doesn't exist."""
+        return self.get(name, scope="preferences")
+
+    def delete_preference(self, name: str) -> None:
+        """delete a preference, it will no longer be available to users."""
+        self.delete(name, scope="preferences")
+
+    def get_preferences(self) -> List[tuple]:
+        """get list of preferences available to users."""
+        return self.list_settings(scope="preferences")
+
     #
     # API for getting at and creating contacts and chats
     #
     @property
     def self_contact(self) -> Contact:
-        """ this bot's contact (with .addr and .display_name attributes). """
+        """this bot's contact (with .addr and .display_name attributes)."""
         return self.account.get_self_contact()
 
     def get_contact(self, ref: Union[str, int, Contact]) -> Contact:
@@ -326,7 +333,7 @@ class DeltaBot:
         ref can be a Message, Contact, email address string or chat-id integer.
         """
         if isinstance(ref, dc.message.Message):
-            return self.account._create_chat_by_message_id(ref.id)
+            return ref.chat
         if isinstance(ref, (dc.contact.Contact, str)):
             return self.account.create_chat(ref)
         if isinstance(ref, int):
@@ -335,20 +342,20 @@ class DeltaBot:
             except ValueError:
                 return None
 
-    def create_group(self, name: str, contacts=[]) -> Chat:
-        """ Create a new group chat. """
-        return self.account.create_group_chat(name, contacts)
+    def create_group(self, name: str, contacts: list = None) -> Chat:
+        """Create a new group chat."""
+        return self.account.create_group_chat(name, contacts or [])
 
     #
     # configuration related API
     #
 
     def is_configured(self) -> bool:
-        """ Return True if this bot account is successfully configured. """
+        """Return True if this bot account is successfully configured."""
         return bool(self.account.is_configured())
 
     def perform_configure_address(self, email: str, password: str) -> bool:
-        """ perform initial email/password bot account configuration.  """
+        """perform initial email/password bot account configuration."""
         assert not self.is_configured() or self.account.get_config("addr") == email
 
         self.account.update_config(
@@ -362,7 +369,7 @@ class DeltaBot:
                 save_mime_headers=1,
                 e2ee_enabled=1,
                 sentbox_watch=0,
-                mvbox_watch=0,
+                mvbox_move=0,
                 bcc_self=0,
                 selfstatus="I'm a Delta Chat bot 🤖. Send me /help for more info.\n\nSource code: https://github.com/simplebot-org/simplebot",
             )
@@ -375,35 +382,63 @@ class DeltaBot:
                 configtracker.wait_finish()
             except configtracker.ConfigureFailed as ex:
                 success = False
-                self.logger.error("Failed to configure: {}".format(ex))
+                self.logger.error("Failed to configure: %s", ex)
             else:
                 set_builtin_avatar(self)
                 success = True
-                self.logger.info("Successfully configured {}".format(email))
+                self.logger.info("Successfully configured %s", email)
             return success
 
     #
     # start/wait/shutdown API
     #
     def start(self) -> None:
-        """ Start bot threads and processing messages. """
+        """Start bot threads and processing messages."""
         self.plugins.hook.deltabot_start(bot=self)
         addr = self.account.get_config("addr")
-        self.logger.info("bot listening at: {}".format(addr))
+        self.logger.info("bot listening at: %s", addr)
         self._eventhandler.start()
         self.account.start_io()
 
     def wait_shutdown(self) -> None:
-        """ Wait and block until bot account is shutdown. """
+        """Wait and block until bot account is shutdown."""
         self.account.wait_shutdown()
         self._eventhandler.stop()
 
     def trigger_shutdown(self) -> None:
-        """ Trigger a shutdown of the bot. """
+        """Trigger a shutdown of the bot."""
         self._eventhandler.stop()
         self.plugins.hook.deltabot_shutdown(bot=self)
         self.account.shutdown()
 
+    #
+    # help command
+    #
+    def _help(self, bot, command, replies) -> None:
+        """get the bot's help."""
+        is_admin = bot.is_admin(command.message.get_sender_contact().addr)
+
+        cmds = []
+        has_prefs = bool(bot.get_preferences())
+        for c in self.commands._cmd_defs.values():
+            if not c.admin or is_admin:
+                if c.cmd != "/set" or has_prefs:
+                    cmds.append(c)
+
+        filters = []
+        for f in self.filters._filter_defs.values():
+            if not f.admin or is_admin:
+                filters.append(f)
+
+        plugins = []
+        for plug, dist in bot.plugins._pm.list_plugin_distinfo():
+            plugins.append(bot.plugins._pm.get_name(plug))
+
+        html = help_template.render(
+            addr=bot.self_contact.addr, cmds=cmds, filters=filters, plugins=plugins
+        )
+        replies.add(text="ℹ️ Help", html=html)
+
 
 class CheckAll:
     def __init__(self, bot, db) -> None:
@@ -412,30 +447,33 @@ class CheckAll:
 
     def perform(self) -> None:
         logger = self.bot.logger
-        logger.info("CheckAll perform-loop start")
+        logger.debug("CheckAll perform-loop start")
         for msg_id in self.db.get_msgs():
             try:
                 message = self.bot.account.get_message_by_id(msg_id)
                 headers = message.get_mime_headers() or dict()
                 if "Chat-Version" in headers or message.is_encrypted():
                     replies = Replies(message, logger=logger)
-                    logger.info(
-                        "processing incoming fresh message id={}".format(message.id)
-                    )
+                    logger.info("processing incoming fresh message id=%s", message.id)
                     if message.is_system_message():
                         self.handle_system_message(message, replies)
                     elif not message.get_sender_contact().is_blocked():
-                        self.bot.plugins.hook.deltabot_incoming_message(
-                            message=message, bot=self.bot, replies=replies
-                        )
+                        if message.is_bot():
+                            self.bot.plugins.hook.deltabot_incoming_bot_message(
+                                message=message, bot=self.bot, replies=replies
+                            )
+                        else:
+                            self.bot.plugins.hook.deltabot_incoming_message(
+                                message=message, bot=self.bot, replies=replies
+                            )
                     replies.send_reply_messages()
                 else:
                     logger.debug("ignoring classic email id=%s", msg_id)
-                logger.info("processing message id={} FINISHED".format(msg_id))
+                logger.info("processing message id=%s FINISHED", msg_id)
             except Exception as ex:
-                logger.exception("processing message={} failed: {}".format(msg_id, ex))
+                logger.exception("processing message=%s failed: %s", msg_id, ex)
             self.db.pop_msg(msg_id)
-        logger.info("CheckAll perform-loop finish")
+        logger.debug("CheckAll perform-loop finish")
 
     def handle_system_message(self, message: Message, replies: Replies) -> None:
         logger = self.bot.logger
@@ -443,7 +481,7 @@ class CheckAll:
         res = parse_system_image_changed(message.text)
         if res:
             actor, deleted = res
-            logger.info("calling hook deltabot_image_changed")
+            logger.debug("calling hook deltabot_image_changed")
             self.bot.plugins.hook.deltabot_image_changed(
                 message=message,
                 replies=replies,
@@ -457,7 +495,7 @@ class CheckAll:
         res = parse_system_title_changed(message.text, message.chat.get_name())
         if res is not None:
             old_title, actor = res
-            logger.info("calling hook deltabot_title_changed")
+            logger.debug("calling hook deltabot_title_changed")
             self.bot.plugins.hook.deltabot_title_changed(
                 message=message,
                 replies=replies,
@@ -471,9 +509,9 @@ class CheckAll:
         res = parse_system_add_remove(message.text)
         if res:
             action, affected, actor = res
-            hook_name = "deltabot_member_{}".format(action)
+            hook_name = f"deltabot_member_{action}"
             meth = getattr(self.bot.plugins.hook, hook_name)
-            logger.info("calling hook {}".format(hook_name))
+            logger.debug("calling hook %s", hook_name)
             meth(
                 message=message,
                 replies=replies,
@@ -484,9 +522,7 @@ class CheckAll:
             )
             return
 
-        logger.info(
-            "ignoring system message id={} text: {}".format(message.id, message.text)
-        )
+        logger.debug("ignoring system message id=%s text: %s", message.id, message.text)
 
 
 class IncomingEventHandler:
@@ -499,7 +535,7 @@ class IncomingEventHandler:
         self._running = True
 
     def start(self) -> None:
-        self.logger.info("starting bot-event-handler THREAD")
+        self.logger.debug("starting bot-event-handler THREAD")
         self.db = self.bot.plugins._pm.get_plugin(name="db")
         self.bot.account.add_account_plugin(self)
         self._thread = t = threading.Thread(
@@ -524,7 +560,7 @@ class IncomingEventHandler:
         # we always accept incoming messages to remove the need  for
         # bot authors to having to deal with deaddrop/contact requests.
         message.create_chat()
-        self.logger.info(
+        self.logger.debug(
             "incoming message from {} id={} chat={} text={!r}".format(
                 message.get_sender_contact().addr,
                 message.id,
@@ -554,8 +590,6 @@ class IncomingEventHandler:
 
     @account_hookimpl
     def ac_message_delivered(self, message: Message) -> None:
-        self.logger.info(
-            "message id={} chat={} delivered to smtp".format(
-                message.id, message.chat.id
-            )
+        self.logger.debug(
+            "message id=%s chat=%s delivered to smtp", message.id, message.chat.id
         )
