@@ -35,7 +35,7 @@ def make_logger(logdir, stdout_loglevel) -> logging.Logger:
 
     log_path = os.path.join(logdir, "bot.log")
     fhandler = logging.handlers.RotatingFileHandler(
-        log_path, backupCount=5, maxBytes=2000000
+        log_path, backupCount=3, maxBytes=2000000
     )
     fhandler.setLevel(logging.DEBUG)
     fhandler.setFormatter(formatter)
@@ -48,6 +48,6 @@ def test_logger_loglevel(capfd, tmpdir):
     logger = make_logger(tmpdir.strpath, stdout_loglevel=logging.INFO)
     logger.info("hello")
     logger.debug("world")
-    out, err = capfd.readouterr()
+    err = capfd.readouterr()[1]
     assert "hello" in err
     assert "world" not in err
